/*
  VisualHash - A Visual Hash Calculator
  Copyright (C) 2005 Dominik Reichl <oss@dominik-reichl.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

using System;
using System.Collections;
using System.Xml;

namespace VisualHash
{
	public class PrivateXmlConfig
	{
		protected ArrayList m_aFields = new ArrayList();
		protected ArrayList m_aValues = new ArrayList();

		public PrivateXmlConfig()
		{
		}

		public uint LoadFromFile(string strFile)
		{
			XmlDocument docConfig = new XmlDocument();
			XmlNode xmlField, xmlValue;
			string str;

			try { docConfig.Load(strFile); }
			catch(Exception) { return 1; }

			XmlElement el = docConfig.DocumentElement;
			if(el == null) return 2;
			if(el.Name != "Configuration") return 3;

			for(int i = 0; i < el.ChildNodes.Count; i++)
			{
				xmlField = el.ChildNodes[i].Attributes.GetNamedItem("Name");
				if(xmlField == null) continue;

				xmlValue = el.ChildNodes[i].Attributes.GetNamedItem("Value");

				str = xmlField.Value; m_aFields.Add(str);
				
				if(xmlValue != null) { str = xmlValue.Value; m_aValues.Add(str); }
				else { str = ""; m_aValues.Add(str); }
			}

			return 0;
		}
		
		public uint SaveToFile(string strFile)
		{
			XmlTextWriter xtw;

			try { xtw = new XmlTextWriter(strFile, null); }
			catch(Exception) { return 1; }

			if(xtw == null) return 2;

			xtw.WriteStartDocument();
			xtw.WriteWhitespace("\r\n");
			xtw.WriteStartElement("Configuration");
			xtw.WriteWhitespace("\r\n");

			for(int i = 0; i < m_aFields.Count; i++)
			{
				xtw.WriteWhitespace("\t");
				xtw.WriteStartElement("Key");
				xtw.WriteAttributeString("Name", (string)m_aFields[i]);
				xtw.WriteAttributeString("Value", (string)m_aValues[i]);
				xtw.WriteEndElement();
				xtw.WriteWhitespace("\r\n");
			}

			xtw.WriteEndElement();
			xtw.WriteWhitespace("\r\n");
			xtw.WriteEndDocument();
			xtw.Close();
			
			return 0;
		}

		public string GetValue(string strField)
		{
			for(int i = 0; i < m_aFields.Count; i++)
				if((string)m_aFields[i] == strField) return (string)m_aValues[i];

			return "";
		}
		
		public void SetValue(string strField, string strValue)
		{
			int iIndex = -1;

			for(int i = 0; i < m_aFields.Count; i++)
				if((string)m_aFields[i] == strField) { iIndex = i; break; }

			if(iIndex == -1)
			{
				m_aFields.Add(strField);
				m_aValues.Add(strValue);
			}
			else
			{
				m_aFields[iIndex] = strField;
				m_aValues[iIndex] = strValue;
			}
		}
	}
}
