/*
  VisualHash - A Visual Hash Calculator
  Copyright (C) 2005 Dominik Reichl <oss@dominik-reichl.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

using System;

namespace VisualHash.ReHashLib
{
	public class ELF32Hash : ReHashModule
	{
		private uint m_elf32 = 0;
		
		public uint GetRawELFValue() { return m_elf32; }

		public ELF32Hash()
		{
		}

		public override bool InitModule()
		{
			strName = "ELF-32";
			strShortName = "ELF-32";
			strRef = "Unix Executable and Linkable Format Specification";
			uHashSize = 32;
			uInternalBufferSize = 32;

			pbFinalHash = new byte[4];
			return true;
		}

		public override void ReleaseModule()
		{
			SecureZeroArray(pbFinalHash);
			pbFinalHash = null;
		}

		public override void InitNewHash()
		{
			m_elf32 = 0;
		}

		public override void UpdateHash(byte[] pbData, ulong uLength, bool bIsLastBlock)
		{
			uint x;

			for(ulong i = 0; i < uLength; i++)
			{
				m_elf32 = (m_elf32 << 4) + pbData[i];

				x = m_elf32 & 0xF0000000u;
				m_elf32 ^= x >> 24;
				m_elf32 &= ~x;
			}
		}

		public override void FinalizeHash()
		{
			Store32H(pbFinalHash, m_elf32);
		}

		public override uint Test()
		{
			byte[] pb;
			byte[] r1 = { 0x09, 0x78, 0xF0, 0x03 };

			pb = StringToAsciiEx("abcabcabcabcabcabcabcabcabcabcabcabcabcabcabc");
			if(!HashAndCompare(pb, r1)) return 1;

			return 0;
		}
	}
}
