/*
  VisualHash - A Visual Hash Calculator
  Copyright (C) 2005 Dominik Reichl <oss@dominik-reichl.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

using System;

namespace VisualHash.ReHashLib
{
	public class FNV32Hash : ReHashModule
	{
		private uint m_fnv32 = 0;

		public FNV32Hash()
		{
		}
		
		public override bool InitModule()
		{
			strName = "FNV-32";
			strShortName = "FNV-32";
			strRef = "";
			uHashSize = 32;
			uInternalBufferSize = 32;

			pbFinalHash = new byte[4];
			return true;
		}

		public override void ReleaseModule()
		{
			SecureZeroArray(pbFinalHash);
			pbFinalHash = null;
		}

		public override void InitNewHash()
		{
			m_fnv32 = 0x811C9DC5u;
		}

		public override void UpdateHash(byte[] pbData, ulong uLength, bool bIsLastBlock)
		{
			for(ulong i = 0; i < uLength; i++)
			{
				m_fnv32 *= 0x01000193u;
				m_fnv32 ^= pbData[i];
			}
		}

		public override void FinalizeHash()
		{
			Store32H(pbFinalHash, m_fnv32);
		}

		public override uint Test()
		{
			byte[] pb;
			byte[] r1 = { 0x53, 0x57, 0xEC, 0x5B };

			pb = StringToAsciiEx("abcabcabcabcabcabcabcabcabcabcabcabcabcabcabc");
			if(!HashAndCompare(pb, r1)) return 1;

			return 0;
		}
	}
}
