/*
  VisualHash - A Visual Hash Calculator
  Copyright (C) 2005 Dominik Reichl <oss@dominik-reichl.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

using System;
using System.Text;
using System.IO;

namespace VisualHash.ReHashLib
{
	/// <summary>
	/// Utility functions (like functions to hash files and report hashes).
	/// </summary>
	public class ReHashTools
	{
		public ReHashTools()
		{
		}

		public static string ReportHashes(HashManager hm, uint uBaseIndent, uint uFlags)
		{
			int i, j;
			ReHashModule rhm;
			StringBuilder sb = new StringBuilder();
			byte[] pbResult;

			for(i = 0; i < hm.m_hashes.Count; i++)
			{
				rhm = hm.GetHashModule(i);
				if(rhm.bEnabled == false) continue;

				sb.Append(rhm.strShortName);

				j = (int)uBaseIndent - rhm.strShortName.Length;
				if(j >= 1) sb.Append(' ', j);
				sb.Append(": ");

				pbResult = rhm.GetFinalHash();
				sb.Append(ReHashModule.FormatHash(pbResult, 4, 5, uBaseIndent + 2, uFlags));
				sb.Append("\r\n");
			}

			sb.Append("\r\n");
			
			return sb.ToString();
		}

		public static bool HashFile(ref HashManager hm, string strFile)
		{
			FileStream fs = null;

			try { fs = new FileStream(strFile, FileMode.Open, FileAccess.Read, FileShare.Read); }
			catch(Exception) { fs = null; }

			if(fs == null) return false;

			BinaryReader br = new BinaryReader(fs);
			byte[] pbData;
			long lFileSize = fs.Length, lToRead;

			hm.InitHashes();

			while(lFileSize != 0)
			{
				lToRead = (lFileSize > 8000) ? 8000 : lFileSize;

				pbData = br.ReadBytes((int)lToRead);
				
				lFileSize -= lToRead;

				if(lFileSize == 0)
					hm.UpdateHashes(pbData, (ulong)lToRead, true);
				else
					hm.UpdateHashes(pbData, (ulong)lToRead, false);
			}

			hm.FinalizeHashes();
			
			br.Close();
			fs.Close();
			
			return true;
		}
	}
}
