/*
  VisualHash - A Visual Hash Calculator
  Copyright (C) 2005 Dominik Reichl <oss@dominik-reichl.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

using System;
using System.Security.Cryptography;

namespace VisualHash.ReHashLib
{
	public class SHA512Hash : ReHashModule
	{
		protected SHA512Managed m_sha512 = new SHA512Managed();

		public SHA512Hash()
		{
		}

		public override bool InitModule()
		{
			strName = "SHA-512";
			strShortName = "SHA-512";
			strRef = "FIPS PUB 180-2";
			uHashSize = 512;
			uInternalBufferSize = 512;

			// pbFinalHash = new byte[64];
			pbFinalHash = null;

			return true;
		}

		public override void ReleaseModule()
		{
			m_sha512.Clear();

			SecureZeroArray(pbFinalHash);
			pbFinalHash = null;
		}

		public override void InitNewHash()
		{
			m_sha512.Initialize();
		}

		public override void UpdateHash(byte[] pbData, ulong uLength, bool bIsLastBlock)
		{
			if(bIsLastBlock) m_sha512.TransformFinalBlock(pbData, 0, (int)uLength);
			else m_sha512.TransformBlock(pbData, 0, (int)uLength, pbData, 0);
		}

		public override void FinalizeHash()
		{
			pbFinalHash = m_sha512.Hash;
		}

		public override uint Test()
		{
			byte[] pb;
			byte[] r1 = { 0x61, 0x23, 0xDA, 0x9A, 0xBD, 0x4F, 0xD2, 0x97,
				0x5C, 0x94, 0xD9, 0x55, 0x50, 0xBC, 0xA6, 0x6C,
				0xA3, 0x44, 0x5E, 0x67, 0x5D, 0x85, 0x21, 0x16,
				0x13, 0xE3, 0xA4, 0xF4, 0xB6, 0xBF, 0x9F, 0xCF,
				0x83, 0x1E, 0x8F, 0x74, 0xDF, 0x3D, 0x10, 0x5A,
				0x4E, 0xCE, 0xED, 0x30, 0xA5, 0x51, 0xF2, 0x76,
				0x92, 0x97, 0x92, 0x62, 0x1A, 0x3A, 0xC2, 0x08,
				0xBB, 0x31, 0x53, 0x37, 0x1B, 0x75, 0x6F, 0x72 };

			pb = StringToAsciiEx("abcabcabcabcabcabcabcabcabcabcabcabcabcabcabc");
			if(!HashAndCompare(pb, r1)) return 1;

			return 0;
		}
	}
}
