/*
  VisualHash - A Visual Hash Calculator
  Copyright (C) 2005 Dominik Reichl <oss@dominik-reichl.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

using System;

namespace VisualHash.ReHashLib
{
	public class Size64Hash : ReHashModule
	{
		private ulong m_size64 = 0;

		public Size64Hash()
		{
		}

		public override bool InitModule()
		{
			strName = "Size-64";
			strShortName = "Size-64";
			strRef = "";
			uHashSize = 64;
			uInternalBufferSize = 64;

			pbFinalHash = new byte[8];
			return true;
		}

		public override void ReleaseModule()
		{
			SecureZeroArray(pbFinalHash);
			pbFinalHash = null;
		}

		public override void InitNewHash()
		{
			m_size64 = 0;
		}

		public override void UpdateHash(byte[] pbData, ulong uLength, bool bIsLastBlock)
		{
			m_size64 += uLength;
		}

		public override void FinalizeHash()
		{
			Store64H(pbFinalHash, m_size64);
		}

		public override uint Test()
		{
			byte[] pb;
			byte[] r1 = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x11 };

			pb = StringToAsciiEx("AAAAAAAAAAAAAAAAA");
			if(!HashAndCompare(pb, r1)) return 1;

			return 0;
		}
	}
}
